@@ -354,7 +354,7 @@ static void updown(private_child_sa_t *this, bool up)
 				 up ? "up" : "down",
 				 policy->my_ts->is_host(policy->my_ts,
 							this->me.addr) ? "-host" : "-client",
-				 this->me.addr->get_family(this->me.addr) == AF_INET ? "" : "-ipv6",
+				 this->me.addr->get_family(this->me.addr) == AF_INET ? "" : "-v6",
 				 this->config->get_name(this->config),
 				 this->iface ? this->iface : "unknown",
 				 this->reqid,
@@ -832,6 +832,16 @@ static status_t update_hosts(private_child_sa_t *this,
 			{
 				policy->other_ts->set_address(policy->other_ts, other);
 			}
+			
+			/* we reinstall the virtual IP to handle interface romaing
+			 * correctly */
+			if (this->virtual_ip)
+			{
+				charon->kernel_interface->del_ip(charon->kernel_interface,
+												 this->virtual_ip);
+				charon->kernel_interface->add_ip(charon->kernel_interface,
+												 this->virtual_ip, me);
+			}
 		
 			/* reinstall updated policies */
 			charon->kernel_interface->add_policy(charon->kernel_interface,
