@@ -6,7 +6,8 @@
  */
 
 /*
- * Copyright (C) 2006 Tobias Brunner, Daniel Roethlisberger
+ * Copyright (C) 2006-2007 Tobias Brunner
+ * Copyright (C) 2006 Daniel Roethlisberger
  * Copyright (C) 2005-2006 Martin Willi
  * Copyright (C) 2005 Jan Hutter
  * Hochschule fuer Technik Rapperswil
@@ -28,6 +29,7 @@
 typedef enum ike_extension_t ike_extension_t;
 typedef enum ike_condition_t ike_condition_t;
 typedef enum ike_sa_state_t ike_sa_state_t;
+typedef enum statistic_t statistic_t;
 typedef struct ike_sa_t ike_sa_t;
 
 #include <library.h>
@@ -94,7 +96,7 @@ enum ike_extension_t {
 enum ike_condition_t {
 	
 	/**
-	 * Connection is natted somewhere
+	 * Connection is natted (or faked) somewhere
 	 */
 	COND_NAT_ANY = (1<<0),
 	
@@ -107,11 +109,32 @@ enum ike_condition_t {
 	 * other is behind NAT
 	 */
 	COND_NAT_THERE = (1<<2),
+	
+	/**
+	 * Faking NAT to enforce UDP encapsulation
+	 */
+	COND_NAT_FAKE = (1<<3),
 
 	/**
-	 * peer is currently not reachable (due missing route, ...)
+	 * peer has ben authenticated using EAP
 	 */
-	COND_STALE = (1<<3),
+	COND_EAP_AUTHENTICATED = (1<<4),
+};
+
+/**
+ * Information and statistics to query from an SA
+ */
+enum statistic_t {
+	
+	/**
+	 * Relative time for scheduled rekeying
+	 */
+	STAT_REKEY_TIME,
+	
+	/**
+	 * Relative time for scheduled reauthentication
+	 */
+	STAT_REAUTH_TIME,
 };
 
 /**
@@ -227,13 +250,6 @@ struct ike_sa_t {
 	 */
 	ike_sa_state_t (*get_state) (ike_sa_t *this);
 	
-	/**
-	 * @brief Get some statistics about this IKE_SA.
-	 *
-	 * @param next_rekeying			when the next rekeying is scheduled
-	 */
-	void (*get_stats)(ike_sa_t *this, u_int32_t *next_rekeying);	
-	
 	/**
 	 * @brief Set the state of the IKE_SA.
 	 *
@@ -250,6 +266,15 @@ struct ike_sa_t {
 	 */
 	char* (*get_name) (ike_sa_t *this);
 	
+	/**
+	 * @brief Get statistic values from the IKE_SA.
+	 *
+	 * @param this			calling object
+	 * @param kind			kind of requested value
+	 * @return				value as integer
+	 */
+	u_int32_t (*get_statistic)(ike_sa_t *this, statistic_t kind);
+	
 	/**
 	 * @brief Get the own host address.
 	 * 
@@ -447,6 +472,96 @@ struct ike_sa_t {
 	 * @param updates		number of pending updates
 	 */
 	void (*set_pending_updates)(ike_sa_t *this, u_int32_t updates);
+
+#ifdef P2P
+	/**
+	 * @brief Get the server reflexive host.
+	 * 
+	 * @param this 			calling object
+	 * @return				server reflexive host
+	 */
+	host_t* (*get_server_reflexive_host) (ike_sa_t *this);
+	
+	/**
+	 * @brief Set the server reflexive host.
+	 * 
+	 * @param this 			calling object
+	 * @param host			server reflexive host
+	 */
+	void (*set_server_reflexive_host) (ike_sa_t *this, host_t *host);
+	
+	/**
+	 * @brief Initiate the mediation of a mediated connection (i.e. initiate a
+	 * P2P_CONNECT exchange).
+	 * 
+	 * @param this 				calling object
+	 * @param mediated_cfg		peer_cfg of the mediated connection
+	 * @return				
+	 * 						- SUCCESS if initialization started
+	 * 						- DESTROY_ME if initialization failed
+	 */
+	status_t (*initiate_mediation) (ike_sa_t *this, peer_cfg_t *mediated_cfg);
+	
+	/**
+	 * @brief Initiate the mediated connection
+	 * 
+	 * @param this 				calling object
+	 * @param me				local endpoint (gets cloned)
+	 * @param other				remote endpoint (gets cloned)
+	 * @param childs			linked list of child_cfg_t of CHILD_SAs (gets cloned)
+	 * @return				
+	 * 						- SUCCESS if initialization started
+	 * 						- DESTROY_ME if initialization failed
+	 */
+	status_t (*initiate_mediated) (ike_sa_t *this, host_t *me, host_t *other,
+			linked_list_t *childs);
+	
+	/**
+	 * @brief Relay data from one peer to another (i.e. initiate a
+	 * P2P_CONNECT exchange).
+	 *
+	 * Data is cloned.
+	 * 
+	 * @param this 				calling object
+	 * @param requester			ID of the requesting peer
+	 * @param session_id		data of the P2P_SESSIONID payload
+	 * @param session_key		data of the P2P_SESSIONKEY payload
+	 * @param endpoints			endpoints
+	 * @param response			TRUE if this is a response
+	 * @return				
+	 * 						- SUCCESS if relay started
+	 * 						- DESTROY_ME if relay failed
+	 */
+	status_t (*relay) (ike_sa_t *this, identification_t *requester, chunk_t session_id,
+			chunk_t session_key, linked_list_t *endpoints, bool response);
+	
+	/**
+	 * @brief Send a callback to a peer.
+	 * 
+	 * Data is cloned.
+	 * 
+	 * @param this				calling object
+	 * @param peer_id			ID of the other peer
+	 * @return
+	 * 						- SUCCESS if response started
+	 * 						- DESTROY_ME if response failed
+	 */
+	status_t (*callback) (ike_sa_t *this, identification_t *peer_id);
+	
+	/**
+	 * @brief Respond to a P2P_CONNECT request.
+	 * 
+	 * Data is cloned.
+	 * 
+	 * @param this				calling object
+	 * @param peer_id			ID of the other peer
+	 * @param session_id		the session ID supplied by the initiator
+	 * @return
+	 * 						- SUCCESS if response started
+	 * 						- DESTROY_ME if response failed
+	 */
+	status_t (*respond) (ike_sa_t *this, identification_t *peer_id, chunk_t session_id);
+#endif /* P2P */
 	
 	/**
 	 * @brief Initiate a new connection.
@@ -749,6 +864,14 @@ struct ike_sa_t {
 	 */
 	status_t (*reestablish) (ike_sa_t *this);
 	
+	/**
+	 * @brief Set the lifetime limit received from a AUTH_LIFETIME notify.
+	 *
+	 * @param this			calling object
+	 * @param lifetime		lifetime in seconds
+	 */
+	void (*set_auth_lifetime)(ike_sa_t *this, u_int32_t lifetime);
+	
 	/**
 	 * @brief Set the virtual IP to use for this IKE_SA and its children.
 	 *
