@@ -157,13 +157,13 @@ static status_t build_id(private_ike_auth_t *this, message_t *message)
 		this->ike_sa->set_my_id(this->ike_sa, me->clone(me));
 	}
 	
-	id = id_payload_create_from_identification(this->initiator, me);
+	id = id_payload_create_from_identification(this->initiator ? ID_INITIATOR : ID_RESPONDER, me);
 	message->add_payload(message, (payload_t*)id);
 	
 	/* as initiator, include other ID if it does not contain wildcards */
 	if (this->initiator && !other->contains_wildcards(other))
 	{
-		id = id_payload_create_from_identification(FALSE, other);
+		id = id_payload_create_from_identification(ID_RESPONDER, other);
 		message->add_payload(message, (payload_t*)id);
 	}
 	return SUCCESS;
@@ -297,6 +297,7 @@ static status_t collect_other_init_data(private_ike_auth_t *this, message_t *mes
 	return NEED_MORE; 
 }
 
+
 /**
  * Implementation of task_t.build to create AUTH payload from EAP data
  */
@@ -320,7 +321,8 @@ static status_t build_auth_eap(private_ike_auth_t *this, message_t *message)
 	if (!this->initiator)
 	{
 		this->ike_sa->set_state(this->ike_sa, IKE_ESTABLISHED);
-		SIG(IKE_UP_SUCCESS, "IKE_SA established between %D[%H]...[%H]%D",
+		SIG(IKE_UP_SUCCESS, "IKE_SA '%s' established between %D[%H]...[%H]%D",
+			this->ike_sa->get_name(this->ike_sa),
 			this->ike_sa->get_my_id(this->ike_sa), 
 			this->ike_sa->get_my_host(this->ike_sa),
 			this->ike_sa->get_other_host(this->ike_sa),
@@ -365,7 +367,8 @@ static status_t process_auth_eap(private_ike_auth_t *this, message_t *message)
 	if (this->initiator)
 	{
 		this->ike_sa->set_state(this->ike_sa, IKE_ESTABLISHED);
-		SIG(IKE_UP_SUCCESS, "IKE_SA established between %D[%H]...[%H]%D",
+		SIG(IKE_UP_SUCCESS, "IKE_SA '%s' established between %D[%H]...[%H]%D",
+			this->ike_sa->get_name(this->ike_sa),
 			this->ike_sa->get_my_id(this->ike_sa), 
 			this->ike_sa->get_my_host(this->ike_sa),
 			this->ike_sa->get_other_host(this->ike_sa),
@@ -518,6 +521,7 @@ static status_t process_r(private_ike_auth_t *this, message_t *message)
 			break;
 		case NOT_FOUND:
 			/* use EAP if no AUTH payload found */
+			this->ike_sa->set_condition(this->ike_sa, COND_EAP_AUTHENTICATED, TRUE);
 			this->eap_auth = eap_authenticator_create(this->ike_sa);
 			break;
 		default:
@@ -573,7 +577,8 @@ static status_t build_r(private_ike_auth_t *this, message_t *message)
 	if (this->peer_authenticated)
 	{
 		this->ike_sa->set_state(this->ike_sa, IKE_ESTABLISHED);
-		SIG(IKE_UP_SUCCESS, "IKE_SA established between %D[%H]...[%H]%D",
+		SIG(IKE_UP_SUCCESS, "IKE_SA '%s' established between %D[%H]...[%H]%D",
+			this->ike_sa->get_name(this->ike_sa),
 			this->ike_sa->get_my_id(this->ike_sa), 
 			this->ike_sa->get_my_host(this->ike_sa),
 			this->ike_sa->get_other_host(this->ike_sa),
@@ -642,6 +647,12 @@ static status_t process_i(private_ike_auth_t *this, message_t *message)
 				case ADDITIONAL_IP6_ADDRESS:
 					/* handled in ike_mobike task */
 					break;
+				case AUTH_LIFETIME:
+					/* handled in ike_auth_lifetime task */
+					break;
+				case P2P_ENDPOINT:
+					/* handled in ike_p2p task */
+					break;
 				default:
 				{
 					if (type < 16383)
@@ -675,7 +686,8 @@ static status_t process_i(private_ike_auth_t *this, message_t *message)
 	}
 	
 	this->ike_sa->set_state(this->ike_sa, IKE_ESTABLISHED);
-	SIG(IKE_UP_SUCCESS, "IKE_SA established between %D[%H]...[%H]%D",
+	SIG(IKE_UP_SUCCESS, "IKE_SA '%s' established between %D[%H]...[%H]%D",
+		this->ike_sa->get_name(this->ike_sa),
 		this->ike_sa->get_my_id(this->ike_sa), 
 		this->ike_sa->get_my_host(this->ike_sa),
 		this->ike_sa->get_other_host(this->ike_sa),
