@@ -6,6 +6,7 @@
  */
 
 /*
+ * Copyright (C) 2007 Tobias Brunner
  * Copyright (C) 2007 Martin Willi
  * Hochschule fuer Technik Rapperswil
  *
@@ -27,12 +28,16 @@ ENUM(task_type_names, IKE_INIT, CHILD_REKEY,
 	"IKE_NATD",
 	"IKE_MOBIKE",
 	"IKE_AUTHENTICATE",
+	"IKE_AUTH_LIFETIME",
 	"IKE_CERT",
 	"IKE_CONFIG",
 	"IKE_REKEY",
 	"IKE_REAUTH",
 	"IKE_DELETE",
 	"IKE_DPD",
+#ifdef P2P
+	"IKE_P2P",
+#endif /* P2P */
 	"CHILD_CREATE",
 	"CHILD_DELETE",
 	"CHILD_REKEY",
