@@ -279,11 +279,11 @@ static void list_certinfos(private_ca_info_t *this, FILE *out, bool utc)
 		fprintf(out, "%#T, until %#T, ", &thisUpdate, utc, &nextUpdate, utc);
 		if (now > nextUpdate)
 		{
-			fprintf(out, "expired (%V ago)\n", &now, &nextUpdate);
+			fprintf(out, "expired (%#V ago)\n", &now, &nextUpdate);
 		}
 		else
 		{
-			fprintf(out, "ok (expires in %V)\n", &now, &nextUpdate);
+			fprintf(out, "ok (expires in %#V)\n", &now, &nextUpdate);
 		}
 		fprintf(out, "    serial:     %#B, %N\n", &serial,
 				cert_status_names, certinfo->get_status(certinfo));
