@@ -13,7 +13,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: pkcs1.c,v 1.17 2006/01/04 21:00:43 as Exp $
+ * RCSID $Id: pkcs1.c 3252 2007-10-06 21:24:50Z andreas $
  */
 
 #include <stddef.h>
@@ -27,7 +27,7 @@
 #include "defs.h"
 #include "mp_defs.h"
 #include "asn1.h"
-#include "oid.h"
+#include <asn1/oid.h>
 #include "log.h"
 #include "pkcs1.h"
 #include "md2.h"
